/* OpResources.h - support resources for non-Interface derived classes */
/* copyleft GPL; Will 18, 2000 - see COPYING for full details */

#ifndef OPRESOURCES_H
#define OPRESOURCES_H

#include <stdio.h>
#include <string.h>


class OpResources
{
	public:
		OpResources ( char * );
		~OpResources();
		char *error() { return  _error; }
		char *fileName() { return _fileName; }	/* resource file name for these resources */
		int isSaveOnExit() { return _appResources.saveOnExit; }
		void setSaveOnExit ( int );
		void save();

		/* user-definable functions returning resources */
		/* <type> className<resourceName>() { return _appResources.className<resourceName>; } */
		int myClass_myNum() { return _appResources.myClass_myNum; };
		void myClass_myNum ( int num ) { _appResources.myClass_myNum = num; }
		char * myClass_myString() { return _appResources.myClass_myString; }
		void myClass_myString ( char *string ) { strncpy ( _appResources.myClass_myString, string, sizeof ( _appResources.myClass_myString ) ); }

	private:
		/* user-definable structure to contain resources */
		typedef struct _AppResources                                                    
		{
			// parameterType className<parameterName>;
			int saveOnExit;
			int myNum;
			char myString[10];	/* example dimension only */
		} AppResources;

		AppResources _appResources;
		long _fileSize;
		char _fileName[64];			/* this resource file name */
		char _opResourceFile[256];	/* resource file name prefixed by path */
		char _error[256];			/* contains initialisation error report or null */
		FILE *_file;

		void initOpResources(); 	/* specify default resources here */
		void getOpResources();
		void setOpResources();
};

/* optional - define single external global instance */
/* (not required for private class-based resources) */
// extern OpResources *opResources; 

#endif


/*
  Notes:
   
  This class may be used either to support public resources available
  throughout an application, or resources private to a class.
  
  The class aims to facilitate the dynamic changing of resource values and also
  the ability to save and restore state information, between instances of an
  application or between instances of a class.
  
  The declaration of the class in user space varies according to whether the
  data it maintains should be considered private to a class, or is publically
  available on a global basis.
  
  The header and implementation files should be copied and renamed for the
  application or class whose resources they support; the names of the class
  methods should also be change to incorporate some reference to the class or
  application supported. This avoids subsequent confusion if more than one set
  of resource files is supported, e.g between an application OpResources method
  and a class-based OpResources method of the same name.
  
  Resource specifications should preceded by the name of the class or
  application using the resource to ensure unique names.
  
  Sample implementations for both public and private use are given below
  
  
  
  Private resource maintenance.
  
  Consider a class 'myClass'. We might rename OpResources.h and OpResources.cc
  to MC_OpResources.h and MC_OpResources.cc; also the 'ifndef' in the header
  will need changing to reflect this, say to MC_OPRESOURCES.
  
  Using a text editor, do a case-sensitive 'search and replace' in both the
  header and implementation files, substituting 'OpResources' with
  'MC_OpResources'.
  
  We now have a class-named version of the OpResources template file, and can
  proceed to 'fill in the blanks'. Using the examples in the MC_OpResources
  class header file and init_MC_OpResources() implementation, simply specify
  what information is to be stored and retrieved on class destruction and
  instantiation.
    
  In the class header file,
  
    #include "MC_OpResources"
	
  and declare a protected instance pointer
  
    protected:
	  MC_OpResources *mcOpResources;
 
  In the calling class constructor, myClass(...) include
	
  	MC_OpResources *resources = new MC_OpResources( "MyClass-OpResources" );
	mcOpResources = resources;
	
  which keeps a global copy after exit from the constructor.
 
  The required resources can then be referenced by 'mcResources->fileName()',
  which returns the name of the class resource file 'myClass-OpResources'.
  
  Other parameters may be stored and retrieved by appropriate set and get
  methods. Exmaples are given in the OpResources class header and
  implementation, based on the notional class 'myClass'.
  
  In the calling class, these methods are referenced via the global resource
  instance pointer 'mcOpResources', so to retrieve the value of 'myInt', we
  would use
  
    someInt = mcOpResources->get_MC_myInt();
	
  and to set its value
  
    mcOpResources->set_MC_myInt ( 10 );
  
  The destructor of the calling class should contain the statement

    delete mcOpResources;
	
  which causes the state of information contained in the _appResources structure
  to be saved if the saveOnExit flag is set (true).



  Public resource maintenance
  
  Consider an application 'myApp'. We might rename OpResources.h and
  OpResources.cc to MA_OpResources.h and MA_OpResources.cc; also the 'ifndef'
  in the header will need changing to reflect this, say to MA_OPRESOURCES.
  
  Using a text editor, do a case-sensitive 'search and replace' in both the
  header and implementation files, substituting 'OpResources' with
  'MA_OpResources'.
  
  We now have an application-named version of the OpResources template file,
  and can proceed to 'fill in the blanks'. Using the examples in the
  MA_OpResources class header file and init_MA_OpResources() implementation,
  simply specify what information is to be stored and retrieved on class
  destruction and instantiation.
   
  In the class header file, declare an externally available instance, e.g. by
  uncommenting the example line provided (make sure it is application-named)
  
    extern MA_OpResources *maOpResources;

  In the class implementation file, prior to the constructor declare an
  application-wide instance of the class
  
  	MA_OpResources *maOpResources = new MA_OpResources( "MyApp-OpResources" );

  In the header file of the calling code
  
    #include "MA_OpResources.h"

  The parameters to be saved and restored should be set in the resource class as
  described under the 'private' use above. Resources may be referenced
  throughout the application via methods of the type
  
    maOpResources->get_MA_myInt();
	maOpResources->set_MA_myInt ( 10 );
	
  for example (depending on how the resources have been named).



  Defining initialisation resources.
  

  The header file.
  
  Resources are specified in this resource header file in the private structure
  _appResources. These resources should be named for the class in which they
  are to be used, avoiding conflict with resources having the same name in
  different classes:
  
    resource name specification: parameterType className<parameterName>

  For example, if class FirstClass has an integer resource '_number' and class
  SecondClass has the same named integer resource, these are specified as
  
    int FirstClass_number;
	int SecondClass_number;
	char * FirstClass_stringResource;
	
  for example.
  
  In the public part of the class, inline methods should be declared to set and
  return the appropriate resource:
 
    type className<resourceName>() { return _appResources.className<resourceName>; };
	  
  Note that these methods are required only if run-time modification of the
  required resources is to be allowed.

  Methods defined in the implementation file may be used if checking is to be
  performed on restored values.
  
  To make the FirstClass_number resource available to applications including
  this header file, we declare the inline public method
  
    int FirstClass_number() { return _appResources.FirstClass_number; };
	
  and for the character string.

	char * FirstClass_stringResource() { return _appResources.FirstClass_stringResource; };


  The implementation file.
  
  Default resource values for all resources specified in the _appResources
  structure in the header file should be declared in the setOpResources()
  method.
  
  Resource defaults are declared here. These will be used by the relevant
  application or class should the initialisation file be missing.
  
  Using the examples above, we can declare
  
    _appResources.FirstClass_number = 64;
	_appResources.FirstClass_stringResource = "some string";
	_appResources.SecondClass_number = 102;
  
  The resource defaults specified in setOpResources() are used to create a new
  application initialisation file if one does not exist. Subsequently, these
  resource specifications are unused, since the resources are re-initialised
  via the getOpResources() method from the resource file, allowing for runtime
  user-initiated changes to be saved.
  
  An example showing the use of numeric and string values follows.
  
  In the private AppResources structure declare the variables
  
    int myClass_myNum;
	char *myClass_myString;

  In the public part of the class header, define appropriate set and get methods
  for these variables:

    int get_myClass_myNum() { return _appResources.myClass_myNum; };
    void set_myClass_myNum ( int num ) { _appResources.myClass_myNum = num; };
	char * get_myClass_myString() { return _appResources.myClass_myString; }
	void set_myClass_myString ( char *string ) { strncpy ( _appResources.myClass_myString, string, sizeof ( _appResources.myClass_myString ) ); }

  Finally, declare the variables to be saved and restored in the method 
  initOpResources(), giving default values:
  
 	_appResources.myClass_myNum = 123;
	strncpy ( _appResources.myClass_myString, "my string",
				sizeof ( _appResources.myClass_myString ) );
			
  Use of the 'sizeof' function ensures the structure contents aren't corrupted
  by an unintentionally too-large string.

  

  Rationale  	
  This class is intended to supplement the resource management facilities
  offered by the LessTif Application Framework. It is not a component part of
  this framework, but extends it. It can function as a stand-alone class to
  allow other applications not developed with the framework to have support
  for initialisation files.

  The problem which this class attempts to address is that resource
  specification for any class not derived from Interface is not supported.
  Thus resource management for operational classes which have no graphical
  interface component in their construction is not available.
	
  The OpResources class is program-specific, hence no default resource
  specifications are provided apart from saveOnExit (which is classless),
  it being impossible to ascertain in advance the operating requirements of
  a particular application. This class therefore serves as a basic structural
  template for initialisation file support.

*/
